'
'  rsclient.bas
'
'  Client DLL for the RSERVER remote data server
'

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Compiler Directives
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Compile DLL
#Dim All
#Register None

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Included Files
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Include "win32api.inc"
#Include "rserver.inc"


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsRegisterServer
'
'  Creates an entry in the user's HOST file if the entry
'  isn't already there.
'
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub rsRegisterServer Alias "rsRegisterServer" (zServer As Asciiz, zIP As Asciiz) Export

   Dim iFF As Long
   Dim WinDir As Asciiz * 300
   Dim iIP As Long

   Host ADDR Trim$(zServer) To iIP

   If iIP = 0 Then
      GetWindowsDirectory WinDir, SizeOf(WinDir) - 1

      On Error Resume Next
      iFF = FreeFile
      Open WinDir + "\HOSTS" For Append As #iFF
      If Err = 0 Then
         Print #iFF, Trim$(zIP) + "      " + Trim$(zServer)
         Close #iFF
      End If
   End If

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsConnect
'
'  Connects to the database engine - Returns 0 on error
'
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsConnect Alias "rsConnect" _
         ( returnSocket As Long, ByVal iPort As Long, zServer As Asciiz ) Export As Long

   Dim hSocket As Long
   On Error Resume Next

   hSocket = FreeFile
   Tcp Open Port iPort At zServer As hSocket
   returnSocket = hSocket
   If Err Then
      Function = %False
   Else
      Function = %True
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsDisconnect
'
'  Closes the connection
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub rsDisconnect Alias "rsDisconnect" _
               ( ByVal hSocket As Long) Export

   Tcp Close hSocket

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsOpen
'
'  Opens a binary file. Pass it the socket value
'  as obtained from wsInit and the file name.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsOpen Alias "rsOpen" _
               (  ByVal hSocket As Long, _
                  zFile As Asciiz) Export As Long

   Dim sOut As String
   Dim sResp As String
   Dim sFile As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   sFile = Trim$(zFile)

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_OPEN_FILE
   rOut.iFile = 0
   rOut.iDataLen = Len(sFile) + 2
   sOut = Space$(Len(rOut))
   Lset sOut = rOut
   sOut = sOut + Mki$(Len(sFile)) + sFile

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   '- Process the result
   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp

   Lset rResp = sResp

   '- Process Response and return
   '  It returns Positive if it could
   '  open the file OR negative if there
   '  was an error. The abs of the return
   '  indicates the DOS error code.
   '
   If rResp.iResult = 0 Then
      Function = rResp.iFile
   Else
      Function = -1 * rResp.iResult
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsClose
'
'  Closes a database.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub rsClose Alias "rsClose" _
            (  ByVal hSocket As Long, ByVal hFile As Integer) Export

   Dim sOut As String
   Dim sResp As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_CLOSE_FILE
   rOut.iFile = hFile
   rOut.iDataLen = 0
   sOut = Space$(Len(rOut))
   Lset sOut = rOut

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   '- Process the result
   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsPut
'
'  Writes data to the remote file.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsPut Alias "rsPut" _
            (  ByVal hSocket As Long, ByVal hFile As Long, _
               ByVal sData As String ) Export As Long

   Dim sOut As String
   Dim sResp As String
   Dim sReturnBuffer As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_PUT
   rOut.iFile = hFile
   rOut.iDataLen = Len(sData)
   sOut = Space$(Len(rOut))

   Lset sOut = rOut
   sOut = sOut + sData

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   MsgBox "Returned from Put Command"

   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp
   Lset rResp = sResp
   MsgBox "Data Recieved"

   Function = rResp.iResult

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsGet
'
'  Reads data from a remote file.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsGet Alias "rsGet" _
            (  ByVal hSocket As Long, ByVal hFile As Long, ByVal iLength As Long, _
               sDataBuffer As String ) Export As Long

   Dim sOut As String
   Dim sResp As String
   Dim sReturnBuffer As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_GET
   rOut.iFile = hFile
   rOut.iDataLen = 4
   sOut = Space$(Len(rOut))

   Lset sOut = rOut
   sOut = sOut + Mkl$(iLength)

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp
   Lset rResp = sResp

   If rResp.iDataLen > 0 Then
      sDataBuffer = Space$(rResp.iDataLen)
      Tcp Recv hSocket, rResp.iDataLen, sDataBuffer
   End If

   Function = rResp.iResult

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsLof
'
'  Gets the length of the file - returns negative of DOS error code
'  if there's an error.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsLof Alias "rsLof" _
            (  ByVal hSocket As Long, ByVal hFile As Long ) Export As Long

   Dim sOut As String
   Dim sResp As String
   Dim sReturnBuffer As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_LOF
   rOut.iFile = hFile
   rOut.iDataLen = 0
   sOut = Space$(Len(rOut))

   Lset sOut = rOut

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp
   Lset rResp = sResp

   If rResp.iDataLen > 0 Then
      sReturnBuffer = Space$(rResp.iDataLen)
      Tcp Recv hSocket, rResp.iDataLen, sReturnBuffer
      Function = Cvl(Left$(sReturnBuffer, 4))
   Else
      Function = rResp.iResult
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  rsSeek
'
'  Moves position in file.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub rsSeek Alias "rsSeek" _
            (  ByVal hSocket As Long, ByVal hFile As Long, ByVal iPos As Long) Export

   Dim sOut As String
   Dim sResp As String
   Dim sReturnBuffer As String
   Dim rOut As XFER_BUFFER_TYPE
   Dim rResp As XFER_BUFFER_TYPE

   '- Prepare the send buffer
   rOut.iCommand = %RSERVER_SEEK
   rOut.iFile = hFile
   rOut.iDataLen = 4
   sOut = Space$(Len(rOut))
   Lset sOut = rOut
   sOut = sOut + Mkl$(iPos)

   '- Send Data and wait for response
   Tcp Send hSocket, sOut

   sResp = Space$(Len(rResp))
   Tcp Recv hSocket, Len(sResp), sResp
   Lset rResp = sResp

End Sub









                     
